/**
* Set rules for how the map overlays
* (information box and legend) will be displayed
* on the page. */
.overlay_legend {
  position: absolute;
  bottom: 0;
  right: 0;
  background: #fff;
  margin-right: 20px;
  font-family: Arial, sans-serif;
  overflow: auto;
  border-radius: 3px;
  opacity: 0.9;
  padding: 10px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
  line-height: 18px;
  margin-bottom: 40px;
}

.overlay_legend i {
  width: 20px;
  height: 15px;
  float: left;
  margin-right: 8px;
  border-style: solid;
  border-width: 1px;
}


.legend_bottom_left {
  position: absolute;
  bottom: 0;
  left: 0;
  margin-left: 20px;
  margin-bottom: 70px;
  background: #fff;
  font-family: Arial, sans-serif;
  overflow: auto;
  border-radius: 3px;
  opacity: 0.9;
  padding: 10px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
  line-height: 18px;
}



/**
* Set rules for how the choropleth
* (information box and legend) will be displayed
* on the page. */
.choropleth_legend {
  position: absolute;
  top: 20px;
  right: 0;
  background: #fff;
  margin-right: 20px;
  font-family: Arial, sans-serif;
  overflow: auto;
  border-radius: 3px;
  opacity: 0.9;
  padding: 10px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
  line-height: 18px;
  margin-bottom: 40px;
}

.choropleth_legend i {
  width: 20px;
  height: 15px;
  float: left;
  margin-right: 8px;
  border-style: solid;
  border-width: 1px;
}


/**
* Set rules for how the choropleth
* (information box and legend) will be displayed
* on the page. */
.top-right {
  position: absolute;
  top: 20px;
  right: 0;
  background: #fff;
  margin-right: 20px;
  font-family: Arial, sans-serif;
  overflow: auto;
  border-radius: 3px;
  opacity: 0.9;
  padding: 10px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
  line-height: 18px;
  margin-bottom: 40px;
}

.top-right i {
  width: 20px;
  height: 15px;
  float: left;
  margin-right: 8px;
  border-style: solid;
  border-width: 1px;
}

/**
* Set rules for how the choropleth
* (information box and legend) will be displayed
* on the page. */
.center-right {
  position: absolute;
  top: 220px;
  right: 0;
  background: #fff;
  margin-right: 20px;
  font-family: Arial, sans-serif;
  overflow: auto;
  border-radius: 3px;
  opacity: 0.9;
  padding: 10px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
  line-height: 18px;
  margin-bottom: 40px;
}

.center-right i {
  width: 20px;
  height: 15px;
  float: left;
  margin-right: 8px;
  border-style: solid;
  border-width: 1px;
}



.bottom-right {
  position: absolute;
  bottom: 0;
  right: 0;
  background: #fff;
  margin-right: 20px;
  font-family: Arial, sans-serif;
  overflow: auto;
  border-radius: 3px;
  opacity: 0.9;
  padding: 10px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
  line-height: 18px;
  margin-bottom: 40px;
}

.bottom-right i {
  width: 20px;
  height: 15px;
  float: left;
  margin-right: 8px;
  border-style: solid;
  border-width: 1px;
}
